/** Discards property keys from the given value. This function returns a shallow Clone. */
export declare function Discard(value: Record<PropertyKey, any>, keys: PropertyKey[]): Record<PropertyKey, any>;
