from _typeshed import Incomplete

class EndOfText(RuntimeError): ...

class Scanner:
    data: Incomplete
    data_length: Incomplete
    start_pos: int
    pos: int
    flags: Incomplete
    last: Incomplete
    match: Incomplete
    def __init__(self, text, flags: int = 0) -> None: ...
    @property
    def eos(self): ...
    def check(self, pattern): ...
    def test(self, pattern): ...
    def scan(self, pattern): ...
    def get_char(self) -> None: ...
