export declare const PatternBoolean = "(true|false)";
export declare const PatternNumber = "(0|[1-9][0-9]*)";
export declare const PatternString = "(.*)";
export declare const PatternNever = "(?!.*)";
export declare const PatternBooleanExact = "^(true|false)$";
export declare const PatternNumberExact = "^(0|[1-9][0-9]*)$";
export declare const PatternStringExact = "^(.*)$";
export declare const PatternNeverExact = "^(?!.*)$";
