import cjsModule from './index.js';

export const createScriptTransformer = cjsModule.createScriptTransformer;
export const createTranspilingRequire = cjsModule.createTranspilingRequire;
export const handlePotentialSyntaxError = cjsModule.handlePotentialSyntaxError;
export const shouldInstrument = cjsModule.shouldInstrument;
