import cjsModule from './index.js';

export const DIFF_DELETE = cjsModule.DIFF_DELETE;
export const DIFF_EQUAL = cjsModule.DIFF_EQUAL;
export const DIFF_INSERT = cjsModule.DIFF_INSERT;
export const Diff = cjsModule.Diff;
export const diff = cjsModule.diff;
export const diffLinesRaw = cjsModule.diffLinesRaw;
export const diffLinesUnified = cjsModule.diffLinesUnified;
export const diffLinesUnified2 = cjsModule.diffLinesUnified2;
export const diffStringsRaw = cjsModule.diffStringsRaw;
export const diffStringsUnified = cjsModule.diffStringsUnified;
