from _typeshed import Incomplete

esc: str
codes: Incomplete
dark_colors: Incomplete
light_colors: Incomplete

def reset_color(): ...
def colorize(color_key, text): ...
def ansiformat(attr, text): ...
