import cjsModule from './index.js';

export const escapePathForRegex = cjsModule.escapePathForRegex;
export const escapeStrForRegex = cjsModule.escapeStrForRegex;
export const replacePathSepForRegex = cjsModule.replacePathSepForRegex;
